/**************************************************************
	melody sdcc
	Copyrights(C)2007  air variable
	http://www.ne.jp/asahi/air/variable/
	
	version 1.00 August,19,2007 available for SDCC compiler

**************************************************************/

/*
port assignment
1	VDD					8	VSS
2	GP5	O LED			7	GP0	(PGD)
3	GP4	(not used)		6	GP1	(PGC)
4	GP3	I SWITCH(VPP)	5	GP2	O	PIEZO_OUT

VDD 2.2V to 5V
GP3 MUST PULLUP 10K OHM
attention:
Do not press GP3 of push switch when Pickit2/ICD2 is active.

*/
//#include <sdcc-lib.h>
#include <pic12f629.h>

// tone Frequency set (as apend of soft delay 4)
#define	Tone_CL  	 19,
#define	Tone_CsL 	 33,
#define	Tone_DL  	 45,
#define	Tone_DsL 	 57,
#define	Tone_EL  	 68,
#define	Tone_FL  	 79,
#define	Tone_FsL 	 89,
#define	Tone_GL  	 99,
#define	Tone_GsL 	108,
#define	Tone_AL  	116,
#define	Tone_AsL 	124,
#define	Tone_BL  	131,
#define	Tone_C   	139,
#define	Tone_Cs  	145,
#define	Tone_D   	152,
#define	Tone_Ds  	158,
#define	Tone_E   	163,
#define	Tone_F   	169,
#define	Tone_Fs  	174,
#define	Tone_G   	178,
#define	Tone_Gs  	183,
#define	Tone_A   	187,
#define	Tone_As  	191,
#define	Tone_B   	195,
#define	Tone_CH  	198,
#define	Tone_CsH 	202,
#define	Tone_DH  	205,
#define	Tone_DsH 	208,
#define	Tone_EH  	211,
#define	Tone_FH  	213,
#define	Tone_FsH 	216,
#define	Tone_GH  	218,
#define	Tone_GsH 	220,
#define	Tone_AH  	222,
#define	Tone_AsH 	224,
#define	Tone_BH  	226,
#define	Rest   	  0,
#define	CMD		255,
#define	END  	0
#define	TEMPO	,

//Length (Midi timebase like 48 tick)
#define	L1		192,	//whole
#define	L2		96,		//half
#define	L3		64,		//triplets half
#define	L4		48,		//quater
#define L4D		72,		//dotted quater
#define	L6		32,		//triplets quater
#define	L8		24,		//eighth
#define	L8D		36,		//dotted eighth
#define	L12		16,		//triplets eighth
#define	L16		12,		//sixteenth
#define	L16D	18,		//dotted sixteenth
#define	L24		8,		//triplets sixteenth
#define	L32		6,		//32th
#define	L32D	9,		//dotted 32th
#define	L48		4,		//triplets 32th
#define	L64		3,		//64th
#define	L96		2,		//triplets 64th
#define	L192	1,		//192th(The minimum unit)

//output port is
//								//		76543210
#define sound_output 	0x04	//gpio2 00000100
#define led_output 		0x20 	//gpio5 00100000
#define keyinput 		0x08 	//gpio3	00001000

//Tempo trim (0 to 4)
#define tempo_multi 8;

//Initialization
//see Configr -> Configuration Bits...
int at 0x2007 __config = 0x3184;

//Global parameter
unsigned char tone;
unsigned char tempocont = 100;//Default TEMPO

//SONG DATA about 350 note 
const unsigned char songdata[]={
	CMD 150 TEMPO	//tempocont (except 255)

		Tone_AsL	L16
		Rest  		L16
		Tone_AsL	L16
		Rest  		L16
		Tone_GsL	L16
		Tone_GL		L16
		Tone_FL		L12
		Rest		L32
		
		Tone_AsL	L16
		Rest		L16
		Tone_AsL	L16
		Rest  		L16
		Tone_GsL	L16
		Tone_GL		L16
		Tone_FL		L12
		Rest		L32
		
		Tone_GL		L16
		Rest		L16
		Tone_GL		L16
		Rest		L16
		Tone_DsL	L16
		Tone_DL		L16
		Tone_CL		L12
		Rest		L32

		Tone_DsL	L16
		Rest		L16
		Tone_DL		L16
		Rest		L16
		Tone_CL		L8
		Rest		L8
//---
		Tone_C   	L8
		Tone_D   	L8
		Tone_Ds  	L16
		Tone_D   	L16
		Tone_C   	L8
				
		Tone_Ds  	L8
		Tone_F   	L8
		Tone_G   	L8
		Tone_F   	L8
		
		Tone_Ds  	L8
		Tone_G   	L8
		Tone_As  	L8
		Tone_Gs  	L16
		Tone_As  	L16
		 
		Tone_G		L8
		Tone_F		L8
		Tone_G		L8
		Rest		L8
//
		Tone_C		L8
		Tone_D		L8
		Tone_Ds		L16
		Tone_D		L16
		Tone_C		L8
		
		Tone_Ds		L8
		Tone_F		L8
		Tone_G		L8
		Tone_F		L8

		Tone_CH		L12
		Rest		L32
		Tone_CH		L12
		Rest		L32
		Tone_As		L8
		Tone_G		L16
		Tone_As		L16
		
		Tone_G		L12
		Rest		L32
		Tone_G		L12
		Rest		L32
		Tone_CH		L8
		Rest		L8
//

		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32
		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32
		
		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32
		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32

		Rest		L8
		Tone_CH		L8
		Tone_As		L12
		Rest		L32
		Tone_As		L12
		Rest		L32
		
		Tone_G		L8
		Tone_As		L8
		Tone_CH		L8
		Rest		L8
//

		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32
		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32
		
		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32
		Tone_CH		L12
		Rest		L32
		Tone_C		L12
		Rest		L32

		Rest		L8
		Tone_CH		L8
		Tone_As		L12
		Rest		L32
		Tone_As		L12
		Rest		L32
		
		Tone_G		L8
		Tone_F		L8
		Tone_G		L8
		Rest		L8
		
//---
		Tone_As		L8
		Rest		L8
		Tone_As		L8
		Rest		L8
		Tone_As		L8
		Rest		L8
		Tone_As		L8
		Rest		L8

		Tone_As		L16
		Rest		L16
		Tone_As		L16
		Rest		L16
		Tone_Gs		L16
		Tone_G		L16
		Tone_F		L12
		Rest		L32
		
		Tone_As		L16
		Rest		L16
		Tone_As		L16
		Rest		L16
		Tone_Gs		L16
		Tone_G		L16
		Tone_F		L12
		Rest		L32

		Tone_Ds		L8
		Tone_F		L8
		Tone_G		L16
		Rest		L16
		Tone_Gs		L16
		Tone_G		L16
		
		Tone_F		L8
		Tone_Ds		L8
		Tone_As		L8
		Rest		L8
//		
		Tone_As		L16
		Rest		L16
		Tone_As		L16
		Rest		L16
		Tone_Gs		L16
		Tone_G		L16
		Tone_F		L12
		Rest		L32
		
		Tone_As		L16
		Rest		L16
		Tone_As		L16
		Rest		L16
		Tone_Gs		L16
		Tone_G		L16
		Tone_F		L12
		Rest		L32

		Tone_G		L16
		Rest		L16
		Tone_G		L16
		Rest		L16
		Tone_Ds		L16
		Tone_D		L16
		Tone_C		L12
		Rest		L32
		
		Tone_Ds		L16
		Rest		L16
		Tone_D		L16
		Rest		L16
		Tone_C		L8
		Rest		L8

	//Rest
		Rest	L4
    //stop
	CMD END	// Music Last of data 
};



//Timer Int process/

static void timer() interrupt 0 
{
    TMR0 = tone;
	if (tone != 0) 	GPIO ^= sound_output;//alternate port 
	T0IF =0;
}

//tempo control
void wtime(unsigned char t0)
{
    unsigned char i,t1,t2;
    for ( i=0 ; i != t0 ;i++)
    {
		t1 = tempo_multi;
		while(t1--){
			t2 = tempocont;
			while ( t2--);
		}
    }
}

void sleep_control(){
	unsigned char tmp=2;
	//power up flash
	while(tmp--){
		GPIO &= ~led_output;//LED flash
		wtime(20);
		GPIO |= led_output;//LED flash off
		wtime(20);
		}
	GIE  = 0;//Int off
	T0IE = 0;//timer interrupt is off
	while ( (GPIO & keyinput)==0 );//wait if key is pushing
	GPIO = GPIO;
	TRISIO = 0xff;	//0b11111111;//GPIO0 is all input
	_asm
	nop
	_endasm;
	WPU  = 0x37;	//0b00110111;//ALL Pin Pull Up(then Low power)
	IOCB = keyinput	;//GPIO0 SW sending &3.10
	GPIF  = 0;//GPIF flag clear
	GPIE  = 1;//sleep wakekup enable
	GIE  = 0;//Interrupt service is disable
	//vector non jump and execute next command
//	SLEEP();//sleep!
	_asm
	sleep
	_endasm;

//	NOP();//Wake up
	_asm
	nop
	_endasm;
	// Wakeup when GPIO0 is LOW 
	GPIE = 0;//GPIO int desable
	GPIF = 0;//GPIO int flag reset
	T0IE = 1;//timer on
	GPIO = GPIO;	// Bug avoid
	TRISIO = ~(sound_output|led_output);
//	WPU  = 0x01;	//0b00000001;//GP0 Pull Up(sw only)
	GIE	 = 1; //Interrupt is All enable	
	GPIO &= ~led_output;//LED flash
	while ( (GPIO & keyinput)==0 );//wait if key is pushing
}
	
main(){

//Beep out is GPIO2, Power Hold is GPIO1
//Power Hold 1:Power is hold 0:Power is off
	unsigned int n=0;
	unsigned char tmp,tmp2;

//	OSCCAL = _READ_OSCCAL_DATA();//OSCCAL AUTO SET
	_asm
	bsf		_STATUS,5  ;ݸ1߰ƾļä
	call	h'3FF'     ;۸ɍŏIԒn͔ޤ
	movwf	_OSCCAL    ;߯÷lOSCCAL;ļä
	bcf		_STATUS,5 
	_endasm;
	
//	OSCCAL = 0x80;		// Please chase to program address of last code
    CMCON  = 7;
//	ANSEL  = 0b00000000;

#if _pic12F675
    ANSEL  = 0;		//when device is 12F675
#endif

    GPIO   = 0x00;	//0b00000000;
	TRISIO =  ~(sound_output|led_output);//LED and piezo (GP2,GP5) &3.10
    TMR0 =0x00 ;
	GIE  = 1;
	PEIE = 1;

    tone =0x00;
//	OPTION = 	//0b00000010; // prescale 8
	OPTION_REG = 0x02;
	T0IE = 1;
	sleep_control();//sleep
	while(1){	
		//command detect
		tmp=songdata[n];
		tmp2=songdata[n+1];
		if (tmp == 255) {	//Command?
			if (tmp2 == 0){	//Tempo or End of data?
				n=0;
				sleep_control();//sleep
			}
			else{
				tempocont = tmp2;	//New tempo change
				n +=2;
			}
		}

		else{
			if ((GPIO&keyinput)==0) sleep_control();// resume
			tone=tmp;
			wtime(tmp2);
			GPIO ^= led_output;//LED flash
			n +=2;		
		}
	}

}
